/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#if defined(__XC)
    #include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
    #include <htc.h>        /* HiTech General Include File */
#endif

#include <stdint.h>        /* For uint8_t definition */
#include <stdbool.h>       /* For true/false definition */

#include "system.h"        /* System funct/params, like osc/peripheral config */
#include "user.h"          /* User funct/params, such as InitApp */
#include "rc5.h"
#include "nunes-somo-14d.h"

/******************************************************************************/
/* User Global Variable Declaration                                           */
/******************************************************************************/

/* i.e. uint8_t <variable_name>; */

/******************************************************************************/
/* Main Program                                                               */
/******************************************************************************/

void main(void)
{
    /* Configure the oscillator for the device */
    ConfigureOscillator();

    /* Initialize I/O and Peripherals for application */
    InitApp();
    __delay_ms(6);
    somo_reset = low;
    arret();

    while(1)
    {
        CLRWDT();

        resetRC5(&temps16ms_0,25,&flagRC5reset);
        if (flagsRC5.error && !flagsmain0.front0) flagsmain0.front0 = high, temps16ms_1 = 0;
        else if (flagsRC5.error && (temps16ms_1 >= 63)) flagsmain0.front0 = low, flagsRC5.error = low;
        led_r = !flagsRC5.error;

        if (!flagsmain1.memo0 && !flagsmain1.memo1) {
            if (getRC5(&flagRC5reset,&getRC5_cmd,inIR_menu_p,0)) {
                flagsmain0.front2 = high;
                if (++file_val <= nb_files) write_SOMO(file_val);
                else file_val = nb_files, write_SOMO(nb_files);
                file_val1 = file_val;
                temps16ms_2 = 0;
            }
            if (getRC5(&flagRC5reset,&getRC5_cmd,inIR_menu_m,0)) {
                flagsmain0.front2 = high;
                if (file_val == 0) write_SOMO(0);
                else if (--file_val) write_SOMO(file_val);
                else write_SOMO(0);
                file_val1 = file_val;
                temps16ms_2 = 0;
            }

            if (getRC5(&flagRC5reset,&getRC5_cmd,IR_adr,0)) { // touche accs rapide des 1er fichiers
                if (getRC5_cmd <= 9) {
                    flagsmain0.front2 = high;
                    file_val = getRC5_cmd;
                    write_SOMO(file_val);
                    temps16ms_2 = 0;
                }
            }

            if (flagsmain0.front2 && (temps16ms_2 >= 32)) flagsmain0.front2 = low, flagsmain1.front0 = high;
            else if (somo_busy && flagsmain1.front0) {
                flagsmain1.front0 = low;
                flagsmain0.memo1 = high;
                temps16ms_2 = 0;
            }
            else if ((temps16ms_2 >= 32) && flagsmain0.memo1) flagsmain0.memo1 = low;
            else if (!flagsmain0.memo1 && !flagsmain0.front2 && !flagsmain1.front0) {
                if (!flagsmain0.front1) {
                    if (in_bruit) temps16ms_2 = 0;
                    else if (!in_bruit && (temps16ms_2 >= t_bruit)) {
                        flagsmain0.front1 = high;
                        write_SOMO(file_val);
                    }
                }
                else if (in_bruit && flagsmain0.front1 && !flagsmain0.front3) {
                    flagsmain0.front3 = high;
                    temps16ms_2 = 0;
                }
                else if ((temps16ms_2 >= duree) && flagsmain0.front3) {
                    flagsmain0.front3 = low;
                    flagsmain0.front1 = low;
                }
            }
        }

        if (getRC5(&flagRC5reset,&getRC5_cmd,inIR_clear,0)) { // stop touche croix
            write_SOMO(0xffff);
            if (flagsmain1.memo0) {
                flagsmain0.front4 = low;
                flagsmain0.front5 = low;
                flagsmain0.front6 = low;
                flagsmain1.memo0 = low;
                temps16ms_2 = 0;
                while (temps16ms_2 < 32) CLRWDT();
            }
        }

        if (!flagsmain1.memo1 && getRC5(&flagRC5reset,&getRC5_cmd,inIR_vert,0)) {
            flagsmain1.memo0 = high;
            if (!file_val1) file_val1 = 1;
        }
        else if (flagsmain1.memo0) {
            if ((temps16ms_2 >= 32) && flagsmain0.front4) flagsmain0.front4 = low, flagsmain0.front5 = high;
            else if ((temps16ms_2 >= 32) && flagsmain0.front6) flagsmain0.front6 = low;
            else if (somo_busy && flagsmain0.front5) flagsmain0.front5 = low, flagsmain0.front6 = high, temps16ms_2 = 0;
            else if (!flagsmain0.front4 && !flagsmain0.front5 && !flagsmain0.front6) {
                flagsmain0.front4 = high;
                if (file_val1 <= nb_files ) write_SOMO(file_val1);
                file_val1++;
                temps16ms_2 = 0;
            }
        }
        if (file_val1 == (nb_files + 2)) file_val1 = 0, flagsmain1.memo0 = low;

        if (getRC5(&flagRC5reset,&getRC5_cmd,inIR_compos,0)) {
            flagsmain1.memo3 = low;
            flagsmain1.memo1 = high;
            flagsmain1.memo2 = high;
            write_SOMO(496);    // diffuser "entrez les dizaines"
            temps1s_0 = 0;
        }
        else if (flagsmain1.memo1) {
            if (temps1s_0 >= t_compos){
                flagsmain1.memo1 = low;
                flagsmain1.memo2 = low;
                flagsmain1.memo3 = low;
            }
            if (flagsmain1.front1) flagsmain1.front1 = low, flagsmain1.memo3 = high, write_SOMO(497);   // diffuser "entrez les units"
            if (getRC5(&flagRC5reset,&getRC5_cmd,IR_adr,0)) {
                if (getRC5_cmd <= 9) {
                    temps1s_0 = 0;
                    if (flagsmain1.memo2) {
                        if (getRC5_cmd <= (nb_files / 10)) {
                            flagsmain1.memo2 = low;
                            flagsmain1.front1 = high;
                            file_val = getRC5_cmd * 10;
                            //write_SOMO(500 + getRC5_cmd);
                        }
                        else write_SOMO(499);   // diffuser "erreur"
                    }
                    else if (flagsmain1.memo3) {
                        if ((file_val + getRC5_cmd) <= nb_files) {
                            file_val += getRC5_cmd;
                            file_val1 = file_val;
                            write_SOMO(file_val);
                            flagsmain1.memo1 = low;
                            flagsmain1.memo3 = low;
                            temps16ms_2 = 0;
                            while (temps16ms_2 < 32) CLRWDT();
                            flagsmain0.front2 = high;
                            temps16ms_2 = 0;
                        }
                        else write_SOMO(499);   // diffuser "erreur"
                    }
                }
            }
        }

        if (getRC5(&flagRC5reset,&getRC5_cmd,inIR_mute,0)) write_SOMO(0xfffe);  // mute

        if (getRC5(&flagRC5reset,&getRC5_cmd,inIR_haut,0)) {    // touche volume +
            if (!volume) volume = 1;
            if (++volume <= 7) write_SOMO(0xfff0 + volume);
            else volume = 7, write_SOMO(0xfff7);
        }
        if (getRC5(&flagRC5reset,&getRC5_cmd,inIR_bas,0)) { // touche volume -
            if (volume) if (--volume >= 1) write_SOMO(0xfff0 + volume);
        }

        if (getRC5(&flagRC5reset,&getRC5_cmd,inIR_rouge,0)) {
            if (flagsmain1.memo1) {
                flagsmain1.memo1 = low;
                flagsmain1.memo2 = low;
                flagsmain1.memo3 = low;
            }
            if (flagsmain1.memo0) {
                flagsmain0.front4 = low;
                flagsmain0.front5 = low;
                flagsmain0.front6 = low;
                flagsmain1.memo0 = low;
            }
            led_r = low;    // allumer led rouge
            somo_reset = high;
            __delay_ms(6);
            somo_reset = low;
            temps16ms_2 = 0;
            while (temps16ms_2 < 63) CLRWDT();   // attente d'environ 1s
            arret();
        }
    }
}
