/*******************************************************************************
*  Name    : Fonctions RC5                                                     *
*  Author  : Nunes Dos Anjos Carlos                                            *
*  Notice  : Copyright (c) 2005 Nunes                                          *
*          : All Rights Reserved                                               *
*  Date    : 28/03/2013                                                        *
*  Version : 1.0                                                               *
*  Notes   : Encoder / dcoder                                                 *
*          : pour le module Nunes                                              *
*******************************************************************************/

/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#if defined(__XC)
    #include <xc.h>        /* XC8 General Include File */
#elif defined(HI_TECH_C)
    #include <htc.h>       /* HiTech General Include File */
#elif defined(__18CXX)
    #include <p18cxxx.h>   /* C18 General Include File */
#endif

#include "user.h"
#include "rc5.h"


// programme destin aux interruptions du TMR2 pour dcoder
inline void it_tmr2_RC5(void)
{
    static unsigned char bitniv1, bitniv2;	// premier 1/2 bit "bitniv1", second 1/2 bit "bitniv2

    compt_demibitRC5++;

    // lecture des tats de chaque demi-bit
    if (!flagsRC5.demibit) { flagsRC5.demibit = high;
        if (!in_IR) bitniv1 = low; else bitniv1 = high; }
    else {
        flagsRC5.demibit = low;
        if (!in_IR) bitniv2 = low; else bitniv2 = high;

        // traitement des tats uniquement aprs la lecture du second 1/2 bit
        if (compt_demibitRC5 < 26) {
            if (bitniv1 == bitniv2) datatmpRC5 = 0;	// erreur de lecture, bit non valide
            else if (bitniv1 && !bitniv2) datatmpRC5 |= 1;	// si lecture correct bit = 1
            if (compt_demibitRC5 < 24) datatmpRC5 <<= 1;	// dcaler le bit si pas le dernier du code
        }
        else if (compt_demibitRC5 == 27) { 	// vrification de la fin pour traiter le message
            if (!(datatmpRC5 & 0x2000)) datatmpRC5 = 0;	// erreur, pas de bit de start. Le message reu est trop court
            else if (bitniv1 != bitniv2) datatmpRC5 = 0;	// erreur, le message reu est trop long
            else if (!bitniv1 && !bitniv2) datatmpRC5 = 0;	// erreur de fin de rception
            else if (bitniv1 && bitniv2) {	// la fin est correct
                dataRC5.val_l = ((unsigned char) datatmpRC5) & 0b00111111;
                dataRC5.val_h = (unsigned char) (datatmpRC5 >> 6);
                datatmpRC5 = 0;
                flagsRC5.rdy = high;	// code RC5 disponible
                flagsRC5.error = low;
            }
        }
        if (!datatmpRC5) {
            T2CONbits.TMR2ON = low;
            PIE1bits.TMR2IE = low;
            compt_demibitRC5 = 0;
            CCP1CONbits.CCP1M = 0b0100;	// CCP3 sur front descendant
            PIR1bits.CCP1IF = low;
            // effacer ccp3ie pour des essais de trame avec l'hyperterminal
            PIE1bits.CCP1IE = high;     // autoriser interruption CCP3 pour vrifier le temps d'un
        }                               // 1/2 bit d'une autre trame
    }
    PIR1bits.TMR2IF = 0;
}


// programme destin aux interruptions de CCP1 pour dcoder
// Synchronisation par vrification du temps du bit de start
// avec le module capture CCP1
inline void it_ccp1_RC5(void)
{
    unsigned int demibit2_t;

    if (!compt_demibitRC5) {
        // initialisation Timer1
        T1CONbits.T1CKPS = 0;	// prescaler  1
        TMR1H = 0, TMR1L = 0;
        T1CONbits.TMR1ON = high;

        compt_demibitRC5 = 1;
    }
    else if (compt_demibitRC5 == 1) {
        demibit2_t = ((unsigned int) CCPR1H) << 8 | CCPR1L;
        if ((demibit2_t > tmax_demibit2_RC5) || (demibit2_t < tmini_demibit2_RC5)) {	// temps 1778us incorrect
            flagsRC5.error = high;
            compt_demibitRC5 = 0;
            CCP1CONbits.CCP1M = 0b0100;
            PIR1bits.CCP1IF = low;
            return;
        }
        else {	// le temps est correct, on lance la tempo
            CCP1CON = 0;	// arrt capture
            PIE1bits.CCP1IE = low;	// ne plus autoriser d'interruption CCP3
            TMR1H = val3_getRC5_TMR1H;	// dure de 1/4 bit 444us pour se positionner au milieu du 1/2 bit
            TMR1L = val2_getRC5_TMR1L;
            PIE1bits.TMR1IE = high;
            PIR1bits.TMR1IF = low;
            //while (!PIR1bits.TMR1IF);
        }
    }
    PIR1bits.CCP1IF = low;
}


// lancement de la lecture du message 
inline void it_tmr1_RC5(void)
{
    TMR2 = 0;
    T2CONbits.TMR2ON = high;
    T1CONbits.TMR1ON = low;
    PIE1bits.TMR1IE = low;

    flagsRC5.rdy = low;		// rception d'une trame RC5 en cours
    flagsRC5.error = high;
    flagsRC5.demibit = low;
    datatmpRC5 = 3, datatmpRC5 <<= 1;	// insrer de bit start

    PIR1bits.TMR2IF = low;
    PIE1bits.TMR2IE = high;
}


// rception du code RC5
unsigned char getRC5(unsigned char *reset, unsigned char *outcmd, unsigned char adr, unsigned char incmd, unsigned char nb)
{
    static unsigned int var0 = 0;
    static unsigned int var1 = 0;
    static unsigned char i = 255;
    static unsigned char tgl = 0;

    if (dataRC5.cmd == 255) return 0;
    if (*reset) var0 = 0, var1 = 0, tgl = 0;

    if ((var0 != dataRC5.val) && (adr == dataRC5.adr) && (incmd == dataRC5.cmd)) {
        *outcmd = dataRC5.cmd, *reset = low, var0 = dataRC5.val,
        tgl = 1; return 1;
    }

    if (!tgl) {
        if (nb) {
            if (var1 != dataRC5.val) i = 0;
            if (!i && (nb != 1)) return 0;
        }
        if (((!nb && (var1 != dataRC5.val)) || (nb && (i <= nb))) &&
        ((adr == dataRC5.adr) && (incmd == 255))) {
            if (nb) i++;
            *outcmd = dataRC5.cmd, *reset = low, var1 = dataRC5.val;
            return 1;
        }
    }
    return 0;
}

void resetRC5(unsigned char *t, unsigned char val_400ms, unsigned char *reset)
{
    static union {
        unsigned char val;
        struct {
            unsigned front0:1;
            unsigned bit1:1;
        };
    } flags = 0;

    if (DataRdyRC5 && !flags.front0) flags.front0 = high, *t = 0;
    if (flags.front0 && *t >= val_400ms)
        DataRdyRC5 = low, flags.front0 = low, *reset = high,
	dataRC5.cmd = 255;
}

// mission du code RC5
void sendRC5(unsigned int code)	// envoi d'un code RC5
{
    unsigned char i, j;
    unsigned int tmpcode;
	
    T1CONbits.TMR1ON = high;

    for (j = 0;j < 2;j++) {	// on envoie le mme code 2 fois
        tmpcode = code;
        T2CONbits.TMR2ON = high;
        for (i = 0;i < 14;i++) {	// envoie des 14 bits du code
            if ((tmpcode & 8192) == 8192) CCP1CONbits.CCP1M = 0;
            else CCPR1L = 14, CCP1CONbits.CCP1M = 0b1100;    // autorisation de la porteuse  36 khz
            TMR1H = val0_sendRC5_TMR1H;
            TMR1L = val0_sendRC5_TMR1L;
            PIR1bits.TMR1IF = low;
            while (!PIR1bits.TMR1IF);	// pendant 889us
            if ((tmpcode & 8192) == 8192) CCPR1L = 14, CCP1CONbits.CCP1M = 0b1100;   // et l'autre priode est l'oppos
            else CCP1CONbits.CCP1M = 0;
            TMR1H = val0_sendRC5_TMR1H;
            TMR1L = val0_sendRC5_TMR1L;
            PIR1bits.TMR1IF = low;
            while (!PIR1bits.TMR1IF); // pendant 889us
            tmpcode <<= 1;  // on fait un dcalage  gauche pour le bit suivant
        }

        CCP1CONbits.CCP1M = 0;
        T2CONbits.TMR2ON = low;

        //if (j == 1) break;
        T1CONbits.T1CKPS = 0b11;    // prescaler timer 1  1/8
        TMR1H = val1_sendRC5_TMR1H;
        TMR1L = val1_sendRC5_TMR1L;
        PIR1bits.TMR1IF = low;
        while (!PIR1bits.TMR1IF); // on temporise 88,886 mS entre chaque trame
        CLRWDT();
        T1CONbits.T1CKPS = 0;    // prescaler timer 1  1
    }
    T1CONbits.TMR1ON = low;
 }
