/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#if defined(__XC)
    #include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
    #include <htc.h>        /* HiTech General Include File */
#endif

#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */

#include "user.h"
#include "rc5.h"
#include "nunes-somo-14d.h"

/******************************************************************************/
/* User Functions                                                             */
/******************************************************************************/

/* <Initialize variables in user.h and insert code for user algorithms.> */

void InitApp(void)
{
    /* TODO Initialize User Ports/Peripherals/Project here */

    /* Setup analog functionality and port direction */

    /* Initialize peripherals */

    /* Enable interrupts */

    //REGISTRE CMCON0 (configuration comparateur)
    CMCON0 = 0b00000111;
    // CM2 / CM1 / CM0 : compateur off, GP0 GP1 GP2 digital

    // REGISTRE ANSEL (ANALOGIQUE/DIGITAL)
    ANSEL = 0;  // en mode I/O
    // ADCS2    b6 : selection conversion horloge AD
    // ADCS1    b5 : idem
    // ADCS0    b4 : idem
    // ANS3     b3 : entre analogique AN3 si ANS3 = 1
    // ANSx     b1  2 : idem
    // ANS0     b0 : idem

    TRISIO = 0b00001100;
    GPIO = 0b00010001;   //initialisation portG

    // Configuration Timer2
    PR2 = pr2val;
    T2CON = init_timer2;

    // Configuration Timer0
    TMR0 = 131;    // 125*256*500ns=16ms, 256-125=131 @8Mhz
    OPTION_REG = 0b00000111; // prescaler timer 0  1/256
    // GPPU	b7 : 0= Rsistance rappel +5V en service

    // initialisation module capture pour la synchronisation avec la rception du code RC5
    CCP1CONbits.CCP1M = 0b0100; // mode capture, CCP1 sur front descendant

    // REGISTRE INTCON (contrle interruptions standard)
    IOCbits.IOC2 = high;    // interruption sur changement de GP3

    // REGISTRE WPU (pin en entre au +5V)
    WPU = 0;
    // WPUx pin pull-up activ si bit = 1

// REGISTRE INTCON (contrle interruptions standard)
    INTCONbits.GIE = high;
    INTCONbits.T0IE = high;
    INTCONbits.PEIE = high;
// GIE      b7 : masque autorisation gnrale interrupt
// PEIE     b6 : masque autorisation gnrale priphriques

// REGISTRE PIE1 (contrle interruptions priphriques)
    PIE1bits.CCP1IE = high;
// TMR2IE   b1 : masque interrupt TMR2
// ADIE     b6 : masque interrupt convertisseur A/D
// EEIE     b7 : masque interrupt criture EEPROM (  voir sinon =1 )

    WDTCONbits.SWDTEN = high;
}

void arret(void)
{
    do {
        CCP1CON = 0; // mode capture off
        T1CONbits.TMR1ON = low;
        T2CONbits.TMR2ON = low;
        INTCONbits.GIE = low;
        INTCONbits.T0IE = low;
        WDTCONbits.SWDTEN = low;
        INTCONbits.GPIF = low;
        INTCONbits.GPIE = high;
        SLEEP();
        INTCONbits.GPIE = low;
        WDTCONbits.SWDTEN = high;
        CCP1CONbits.CCP1M = 0b0100; // mode capture, CCP1 sur front descendant
        INTCONbits.T0IE = high;
        flagsRC5.rdy = low;
        dataRC5.adr = 0;
        INTCONbits.GIE = high;
        temps16ms_2 = 0;
        while (!flagsRC5.rdy) if (temps16ms_2 >= 63) break;
    }while (dataRC5.adr != 7);
    led_r = high;
    volume = init_volume;
    write_SOMO(0xfff0 + volume);
}

inline void it_tmr0_time(void)
{
    temps16ms_0++;
    temps16ms_1++;
    temps16ms_2++;
    if (++temps16ms_3 >= 63) temps1s_0++, temps16ms_3 = 0;
    TMR0 = 131;    // 125*256*500ns=16ms, 256-125=131 @8Mhz
    INTCONbits.T0IF = low;
}
